@@ -1,37 +1,43 @@
-# -*- coding: latin-1 -*-
-
-"""
-
-Calendar is a dictionary like Python object that can render itself as VCAL
+# -*- coding: utf-8 -*-
+"""Calendar is a dictionary like Python object that can render itself as VCAL
 files according to rfc2445.
 
 These are the defined components.
 
 """
 
-# from python
+import pytz
+from datetime import datetime
 from types import ListType, TupleType
-SequenceTypes = (ListType, TupleType)
-
-# from this package
 from icalendar.caselessdict import CaselessDict
-from icalendar.parser import Contentlines, Contentline, Parameters
-from icalendar.parser import q_split, q_join
-from icalendar.prop import TypesFactory, vText
+from icalendar.parser import (
+    Contentlines,
+    Contentline,
+    Parameters,
+)
+from icalendar.parser import (
+    q_split,
+    q_join,
+)
+from icalendar.prop import (
+    TypesFactory,
+    vText,
+)
+
+SequenceTypes = (ListType, TupleType)
 
 
 ######################################
 # The component factory
 
 class ComponentFactory(CaselessDict):
-    """
-    All components defined in rfc 2445 are registered in this factory class. To
-    get a component you can use it like this.
+    """ All components defined in rfc 2445 are registered in this factory
+    class. To get a component you can use it like this.
 
     >>> factory = ComponentFactory()
     >>> component = factory['VEVENT']
     >>> event = component(dtstart='19700101')
-    >>> event.as_string()
+    >>> event.to_ical()
     'BEGIN:VEVENT\\r\\nDTSTART:19700101\\r\\nEND:VEVENT\\r\\n'
 
     >>> factory.get('VCALENDAR', Component)
@@ -103,7 +109,7 @@ class Component(CaselessDict):
     >>> c = Component()
     >>> c.name = 'VCALENDAR'
     >>> c.add('attendee', 'Max M')
-    >>> c.as_string()
+    >>> c.to_ical()
     'BEGIN:VCALENDAR\\r\\nATTENDEE:Max M\\r\\nEND:VCALENDAR\\r\\n'
 
     >>> from icalendar.prop import vDatetime
@@ -113,7 +119,7 @@ class Component(CaselessDict):
     >>> e.name = 'VEVENT'
     >>> e.add('dtend', '20000102T000000', encode=0)
     >>> e.add('dtstart', '20000101T000000', encode=0)
-    >>> e.as_string()
+    >>> e.to_ical()
     'BEGIN:VEVENT\\r\\nDTEND:20000102T000000\\r\\nDTSTART:20000101T000000\\r\\nSUMMARY:A brief history of time\\r\\nEND:VEVENT\\r\\n'
 
     >>> c.add_component(e)
@@ -132,11 +138,22 @@ class Component(CaselessDict):
     >>> [i['dtstart'] for i in c.walk('VEVENT')]
     ['20000101T000000']
 
+    We can enumerate property items recursively with the property_items method.
+    >>> c.property_items()
+    [('BEGIN', 'VCALENDAR'), ('ATTENDEE', vCalAddress('Max M')), ('BEGIN', 'VEVENT'), ('DTEND', '20000102T000000'), ('DTSTART', '20000101T000000'), ('SUMMARY', 'A brief history of time'), ('END', 'VEVENT'), ('END', 'VCALENDAR')]
+
+    We can also enumerate property items just under the component.
+    >>> c.property_items(recursive=False)
+    [('BEGIN', 'VCALENDAR'), ('ATTENDEE', vCalAddress('Max M')), ('END', 'VCALENDAR')]
+    >>> sc = c.subcomponents[0]
+    >>> sc.property_items(recursive=False)
+    [('BEGIN', 'VEVENT'), ('DTEND', '20000102T000000'), ('DTSTART', '20000101T000000'), ('SUMMARY', 'A brief history of time'), ('END', 'VEVENT')]
+
     Text fields which span multiple mulitple lines require proper indenting
     >>> c = Calendar()
     >>> c['description']=u'Paragraph one\\n\\nParagraph two'
-    >>> c.as_string()
-    'BEGIN:VCALENDAR\\r\\nDESCRIPTION:Paragraph one\\r\\n \\r\\n Paragraph two\\r\\nEND:VCALENDAR\\r\\n'
+    >>> c.to_ical()
+    'BEGIN:VCALENDAR\\r\\nDESCRIPTION:Paragraph one\\n\\nParagraph two\\r\\nEND:VCALENDAR\\r\\n'
 
     INLINE properties have their values on one property line. Note the double
     quoting of the value with a colon in it.
@@ -145,7 +162,7 @@ class Component(CaselessDict):
     >>> c
     VCALENDAR({'RESOURCES': 'Chair, Table, "Room: 42"'})
 
-    >>> c.as_string()
+    >>> c.to_ical()
     'BEGIN:VCALENDAR\\r\\nRESOURCES:Chair, Table, "Room: 42"\\r\\nEND:VCALENDAR\\r\\n'
 
     The inline values must be handled by the get_inline() and set_inline()
@@ -194,7 +211,6 @@ class Component(CaselessDict):
         self.subcomponents = [] # Components can be nested.
         self.is_broken = False  # True iff we ignored an exception while parsing a property
 
-
 #    def non_complience(self, warnings=0):
 #        """
 #        not implemented yet!
@@ -210,27 +226,62 @@ class Component(CaselessDict):
 #            nc['name'] = {'type':'ERROR', 'description':'Name is not defined'}
 #        return nc
 
-
     #############################
     # handling of property values
 
     def _encode(self, name, value, cond=1):
-        # internal, for conditional convertion of values.
+        """Conditional convertion of values.
+
+        """
         if cond:
             klass = types_factory.for_property(name)
-            return klass(value)
+            _klass = klass(value)
+            if hasattr(value, 'params') and len(value.params.keys()) > 0:
+                _klass.params = value.params
+            return _klass
         return value
 
-
     def set(self, name, value, encode=1):
         if type(value) == ListType:
             self[name] = [self._encode(name, v, encode) for v in value]
         else:
             self[name] = self._encode(name, value, encode)
 
-
     def add(self, name, value, encode=1):
-        "If property exists append, else create and set it"
+        """ Add a property.
+
+        Test the for timezone correctness: dtstart should preserve it's
+        timezone, crated, dtstamp and last-modified must be in UTC.
+        >>> from datetime import datetime
+        >>> import pytz
+        >>> from icalendar.cal import Component
+        >>> comp = Component()
+        >>> comp.add('dtstart', datetime(2010,10,10,10,0,0,tzinfo=pytz.timezone("Europe/Vienna")))
+        >>> comp.add('created', datetime(2010,10,10,12,0,0))
+        >>> comp.add('dtstamp', datetime(2010,10,10,14,0,0,tzinfo=pytz.timezone("Europe/Vienna")))
+        >>> comp.add('last-modified', datetime(2010,10,10,16,0,0,tzinfo=pytz.utc))
+
+        >>> lines = comp.to_ical().splitlines()
+        >>> "DTSTART;TZID=Europe/Vienna;VALUE=DATE-TIME:20101010T100000" in lines
+        True
+        >>> "CREATED;VALUE=DATE-TIME:20101010T120000Z" in lines
+        True
+        >>> "DTSTAMP;VALUE=DATE-TIME:20101010T130000Z" in lines
+        True
+        >>> "LAST-MODIFIED;VALUE=DATE-TIME:20101010T160000Z" in lines
+        True
+
+        """
+        if isinstance(value, datetime) and\
+                name.lower() in ('dtstamp', 'created', 'last-modified'):
+            # RFC expects UTC for those... force value conversion.
+            if getattr(value, 'tzinfo', False) and value.tzinfo is not None:
+                value = value.astimezone(pytz.utc)
+            else:
+                # assume UTC for naive datetime instances
+                value = pytz.utc.localize(value)
+
+        # If property exists append, else create and set it.
         if name in self:
             oldval = self[name]
             value = self._encode(name, value, encode)
@@ -240,14 +291,16 @@ class Component(CaselessDict):
                 self.set(name, [oldval, value], encode=0)
         else:
             self.set(name, value, encode)
-
+        if getattr(value, 'tzinfo', False) and value.tzinfo is not None and value.tzinfo is not pytz.utc:
+            # set the timezone as a parameter to the property
+            tzid = value.tzinfo.zone
+            self[name].params.update({'TZID': tzid})
 
     def _decode(self, name, value):
         # internal for decoding property values
         decoded = types_factory.from_ical(name, value)
         return decoded
 
-
     def decoded(self, name, default=_marker):
         "Returns decoded value of property"
         if name in self:
@@ -261,7 +314,6 @@ class Component(CaselessDict):
             else:
                 return default
 
-
     ########################################################################
     # Inline values. A few properties have multiple values inlined in in one
     # property line. These methods are used for splitting and joining these.
@@ -276,7 +328,6 @@ class Component(CaselessDict):
             return [self._decode(name, val) for val in vals]
         return vals
 
-
     def set_inline(self, name, values, encode=1):
         """
         Converts a list of values into comma seperated string and sets value to
@@ -287,7 +338,6 @@ class Component(CaselessDict):
         joined = q_join(values).encode(vText.encoding)
         self[name] = types_factory['inline'](joined)
 
-
     #########################
     # Handling of components
 
@@ -295,7 +345,6 @@ class Component(CaselessDict):
         "add a subcomponent to this component"
         self.subcomponents.append(component)
 
-
     def _walk(self, name):
         # private!
         result = []
@@ -305,7 +354,6 @@ class Component(CaselessDict):
             result += subcomponent._walk(name)
         return result
 
-
     def walk(self, name=None):
         """
         Recursively traverses component and subcomponents. Returns sequence of
@@ -318,15 +366,14 @@ class Component(CaselessDict):
     #####################
     # Generation
 
-    def property_items(self):
+    def property_items(self, recursive=True):
         """
         Returns properties in this component and subcomponents as:
         [(name, value), ...]
         """
         vText = types_factory['text']
-        properties = [('BEGIN', vText(self.name).ical())]
-        property_names = self.keys()
-        property_names.sort()
+        properties = [('BEGIN', vText(self.name).to_ical())]
+        property_names = self.sorted_keys()
         for name in property_names:
             values = self[name]
             if type(values) == ListType:
@@ -335,20 +382,31 @@ class Component(CaselessDict):
                     properties.append((name, value))
             else:
                 properties.append((name, values))
-        # recursion is fun!
-        for subcomponent in self.subcomponents:
-            properties += subcomponent.property_items()
-        properties.append(('END', vText(self.name).ical()))
+        if recursive:
+            # recursion is fun!
+            for subcomponent in self.subcomponents:
+                properties += subcomponent.property_items()
+        properties.append(('END', vText(self.name).to_ical()))
         return properties
 
-
-    def from_string(st, multiple=False):
+    def from_ical(st, multiple=False):
         """
         Populates the component recursively from a string
+
+        RecurrenceIDs may contain a TZID parameter, if so, they should create a tz localized datetime, otherwise, create a naive datetime
+        >>> componentStr = 'BEGIN:VEVENT\\nRECURRENCE-ID;TZID=America/Denver:20120404T073000\\nEND:VEVENT'
+        >>> component = Component.from_ical(componentStr)
+        >>> component['RECURRENCE-ID'].dt.tzinfo
+        <DstTzInfo 'America/Denver' MDT-1 day, 18:00:00 DST>
+
+        >>> componentStr = 'BEGIN:VEVENT\\nRECURRENCE-ID:20120404T073000\\nEND:VEVENT'
+        >>> component = Component.from_ical(componentStr)
+        >>> component['RECURRENCE-ID'].dt.tzinfo == None
+        True
         """
         stack = [] # a stack of components
         comps = []
-        for line in Contentlines.from_string(st): # raw parsing
+        for line in Contentlines.from_ical(st): # raw parsing
             if not line:
                 continue
             name, params, vals = line.parts()
@@ -378,7 +436,10 @@ class Component(CaselessDict):
                 factory = types_factory.for_property(name)
                 component = stack[-1]
                 try:
-                    vals = factory(factory.from_ical(vals))
+                    if name in ('DTSTART', 'DTEND','RECURRENCE-ID') and 'TZID' in params: # TODO: add DUE, FREEBUSY
+                        vals = factory(factory.from_ical(vals, params['TZID']))
+                    else:
+                        vals = factory(factory.from_ical(vals))
                 except ValueError:
                     if not component.ignore_exceptions:
                         raise
@@ -396,8 +457,7 @@ class Component(CaselessDict):
             raise ValueError('Found no components where '
                              'exactly one is required: {st!r}'.format(**locals()))
         return comps[0]
-    from_string = staticmethod(from_string)
-
+    from_ical = staticmethod(from_ical)
 
     def __repr__(self):
         return '%s(' % self.name + dict.__repr__(self) + ')'
@@ -417,30 +477,28 @@ class Component(CaselessDict):
         contentlines.append('') # remember the empty string in the end
         return contentlines
 
-
-    def as_string(self):
-        return str(self.content_lines())
-
-
-    def __str__(self):
-        "Returns rendered iCalendar"
-        return self.as_string()
-
+    def to_ical(self):
+        return self.content_lines().to_ical()
 
 
 #######################################
 # components defined in RFC 2445
 
-
 class Event(Component):
 
     name = 'VEVENT'
 
+    canonical_order = (
+        'SUMMARY', 'DTSTART', 'DTEND', 'DURATION', 'DTSTAMP',
+        'UID', 'RECURRENCE-ID', 'SEQUENCE',
+        'RRULE' 'EXRULE', 'RDATE', 'EXDATE',
+    )
+
     required = ('UID',)
     singletons = (
         'CLASS', 'CREATED', 'DESCRIPTION', 'DTSTART', 'GEO',
-        'LAST-MOD', 'LOCATION', 'ORGANIZER', 'PRIORITY', 'DTSTAMP', 'SEQUENCE',
-        'STATUS', 'SUMMARY', 'TRANSP', 'URL', 'RECURID', 'DTEND', 'DURATION',
+        'LAST-MODIFIED', 'LOCATION', 'ORGANIZER', 'PRIORITY', 'DTSTAMP', 'SEQUENCE',
+        'STATUS', 'SUMMARY', 'TRANSP', 'URL', 'RECURRENCE-ID', 'DTEND', 'DURATION',
         'DTSTART',
     )
     exclusive = ('DTEND', 'DURATION', )
@@ -458,8 +516,8 @@ class Todo(Component):
     required = ('UID',)
     singletons = (
         'CLASS', 'COMPLETED', 'CREATED', 'DESCRIPTION', 'DTSTAMP', 'DTSTART',
-        'GEO', 'LAST-MOD', 'LOCATION', 'ORGANIZER', 'PERCENT', 'PRIORITY',
-        'RECURID', 'SEQUENCE', 'STATUS', 'SUMMARY', 'UID', 'URL', 'DUE', 'DURATION',
+        'GEO', 'LAST-MODIFIED', 'LOCATION', 'ORGANIZER', 'PERCENT', 'PRIORITY',
+        'RECURRENCE-ID', 'SEQUENCE', 'STATUS', 'SUMMARY', 'UID', 'URL', 'DUE', 'DURATION',
     )
     exclusive = ('DUE', 'DURATION',)
     multiple = (
@@ -468,15 +526,14 @@ class Todo(Component):
     )
 
 
-
 class Journal(Component):
 
     name = 'VJOURNAL'
 
     required = ('UID',)
     singletons = (
-        'CLASS', 'CREATED', 'DESCRIPTION', 'DTSTART', 'DTSTAMP', 'LAST-MOD',
-        'ORGANIZER', 'RECURID', 'SEQUENCE', 'STATUS', 'SUMMARY', 'UID', 'URL',
+        'CLASS', 'CREATED', 'DESCRIPTION', 'DTSTART', 'DTSTAMP', 'LAST-MODIFIED',
+        'ORGANIZER', 'RECURRENCE-ID', 'SEQUENCE', 'STATUS', 'SUMMARY', 'UID', 'URL',
     )
     multiple = (
         'ATTACH', 'ATTENDEE', 'CATEGORIES', 'COMMENT', 'CONTACT', 'EXDATE',
@@ -499,12 +556,13 @@ class FreeBusy(Component):
 class Timezone(Component):
 
     name = 'VTIMEZONE'
+    canonical_order = ('TZID', 'STANDARDC', 'DAYLIGHTC',)
 
     required = (
         'TZID', 'STANDARDC', 'DAYLIGHTC', 'DTSTART', 'TZOFFSETTO',
         'TZOFFSETFROM'
         )
-    singletons = ('LAST-MOD', 'TZURL', 'TZID',)
+    singletons = ('LAST-MODIFIED', 'TZURL', 'TZID',)
     multiple = ('COMMENT', 'RDATE', 'RRULE', 'TZNAME',)
 
 
@@ -536,13 +594,13 @@ class Calendar(Component):
     >>> event['uid'] = '42'
     >>> event.set('dtstart', datetime(2005,4,4,8,0,0))
     >>> cal.add_component(event)
-    >>> cal.subcomponents[0].as_string()
-    'BEGIN:VEVENT\\r\\nDTSTART;VALUE=DATE:20050404T080000\\r\\nSUMMARY:Python meeting about calendaring\\r\\nUID:42\\r\\nEND:VEVENT\\r\\n'
+    >>> cal.subcomponents[0].to_ical()
+    'BEGIN:VEVENT\\r\\nSUMMARY:Python meeting about calendaring\\r\\nDTSTART;VALUE=DATE-TIME:20050404T080000\\r\\nUID:42\\r\\nEND:VEVENT\\r\\n'
 
     Write to disc
     >>> import tempfile, os
     >>> directory = tempfile.mkdtemp()
-    >>> open(os.path.join(directory, 'test.ics'), 'wb').write(cal.as_string())
+    >>> open(os.path.join(directory, 'test.ics'), 'wb').write(cal.to_ical())
 
     Parsing a complete calendar from a string will silently ignore bogus events.
     The bogosity in the following is the third EXDATE: it has an empty DATE.
@@ -567,16 +625,16 @@ class Calendar(Component):
     ...                    'EXDATE;VALUE=DATE:',
     ...                    'END:VEVENT',
     ...                    'END:VCALENDAR'))
-    >>> [e['DESCRIPTION'].ical() for e in Calendar.from_string(s).walk('VEVENT')]
+    >>> [e['DESCRIPTION'].to_ical() for e in Calendar.from_ical(s).walk('VEVENT')]
     ['Perfectly OK event']
     """
 
     name = 'VCALENDAR'
+    canonical_order = ('VERSION', 'PRODID', 'CALSCALE', 'METHOD',)
     required = ('prodid', 'version', )
     singletons = ('prodid', 'version', )
     multiple = ('calscale', 'method', )
 
-
 # These are read only singleton, so one instance is enough for the module
 types_factory = TypesFactory()
 component_factory = ComponentFactory()
